<?php

namespace TDevAgency\FilesManager\Http\Controllers;

use Illuminate\Contracts\Filesystem\FileNotFoundException;
use Illuminate\Filesystem\Filesystem;
use Illuminate\Foundation\Auth\Access\AuthorizesRequests;
use Illuminate\Foundation\Bus\DispatchesJobs;
use Illuminate\Foundation\Validation\ValidatesRequests;
use Illuminate\Routing\Controller as BaseController;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Response;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\View;
use TDevAgency\FilesManager\Facades\FmImage;
use TDevAgency\FilesManager\Helpers\CacheHelper;
use TDevAgency\FilesManager\Helpers\GenerateSizesHelper;
use TDevAgency\FilesManager\Helpers\SymlinkHelper;
use TDevAgency\FilesManager\ResourceModels\TdfmFile;

class FilesManagerController extends BaseController
{
    use AuthorizesRequests;
    use DispatchesJobs;
    use ValidatesRequests;

    /**
     * @var Filesystem
     */
    private $fs;

    /** @var GenerateSizesHelper */
    private $generateSizesHelper;

    /** @var SymlinkHelper */
    private $symlinkHelper;
    /** @var CacheHelper */
    private $cacheHelper;

    public function __construct(
        Filesystem $fs,
        GenerateSizesHelper $generateSizesHelper,
        SymlinkHelper $symlinkHelper,
        CacheHelper $cacheHelper,
    ) {
        $this->fs = $fs;
        $this->generateSizesHelper = $generateSizesHelper;
        $this->symlinkHelper = $symlinkHelper;
        $this->cacheHelper = $cacheHelper;
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param $size
     * @param $id
     * @param $file
     * @return \Illuminate\Http\Response
     * @throws FileNotFoundException
     */
    public function image($size, $id, $file)
    {
        if (!config('tdfm.generate_webp') && preg_match('/^(\w+)(.webp)$/', $file)) {
            return Response::make('', 404);
        }
        $tdfmFile = app(CacheHelper::class)->getFileData($id);
        if ($tdfmFile !== null) {
            $path = sprintf(
                '%s' . DS . '%s' . DS . '%s' . DS . '%s',
                Config::get('tdfm.disks.tdfm_generated.root'),
                $id,
                $size,
                $tdfmFile->name
            );
            /** @var File $fileString */
            if ($this->fs->exists($path)) {
            } elseif (Storage::disk('tdfm_original')->exists($tdfmFile->name)) {
                $this->generateSizesHelper->handle($tdfmFile);
                $this->symlinkHelper->symlinkFile($tdfmFile);
                $this->cacheHelper->removeFileData($id);
                $tdfmFile = app(CacheHelper::class)->getFileData($id);
                $path = sprintf(
                    '%s' . DS . '%s' . DS . '%s' . DS . '%s',
                    Config::get('tdfm.disks.tdfm_generated.root'),
                    $id,
                    $size,
                    $tdfmFile->name
                );
            } else {
                return response(null, 404);
            }
            $expires = Carbon::now()->addMonth(6);
            $modified = Carbon::createFromTimestamp($this->fs->lastModified($path));
            return Response::make($this->fs->get($path), 200, ['Content-Type' => $this->fs->mimeType($path)])
                ->setMaxAge(15552000)
                ->setLastModified($modified)
                ->setExpires($expires);
        }
        return response(null, 404);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param $size
     * @param $file
     * @return \Illuminate\Http\Response
     * @throws FileNotFoundException
     */
    public function originalFile($id, $file)
    {
        if (!config('tdfm.generate_webp') && preg_match('/^(\w+)(.webp)$/', $file)) {
            return Response::make('', 404);
        }
        /** @var TdfmFile $tdfmFile */
        $tdfmFile = app(CacheHelper::class)->getFileData($id);
        if ($tdfmFile !== null) {
            $path = sprintf('%s' . DS . '%s', Config::get('tdfm.disks.tdfm_original.root'), $tdfmFile->name);
            /** @var File $fileString */
            if ($this->fs->exists($path)) {
            } elseif (Storage::disk('tdfm_original')->exists($tdfmFile->name)) {
//                $this->generateSizes($tdfmFile);
            } else {
                return response(null, 404);
            }
            $expires = Carbon::now()->addMonth(6);
            $modified = Carbon::createFromTimestamp($this->fs->lastModified($path));
            return Response::make($this->fs->get($path), 200, ['Content-Type' => $tdfmFile->file_mime])
                ->setMaxAge(15552000)
                ->setLastModified($modified)
                ->setExpires($expires);
        }
        return Response::make('', 404);
    }

    /**
     * @return mixed
     */
    public function js()
    {
        $js = View::make('files-manager::js')->render();
        $modified = Carbon::createFromTimestamp(
            $this->fs->lastModified(TDFM_ROOT . 'resources' . DS . 'views' . DS . 'js.blade.php')
        );
        $expires = Carbon::now()->addMonth(6);
        return Response::make($js, 200, ['Content-Type' => 'application/javascript'])
            ->setMaxAge(15552000)
            ->setLastModified($modified)
            ->setExpires($expires);
    }

    /**
     * @param $path1
     * @param $id
     * @param null $path2
     * @return mixed
     * @throws FileNotFoundException
     */
    public function svg_image($path1, $id, $path2 = null)
    {
        $svgName = $path2 ?? $path1;

        $tdfmFile = app(CacheHelper::class)->getFileData($id);
        if ($tdfmFile === null) {
            return Response::make('', 404);
        }
        $path = sprintf('%s' . DS . '%s', Config::get('tdfm.disks.tdfm_original.root'), $tdfmFile->name);

        $expires = Carbon::now()->addMonth(6);
        $modified = Carbon::createFromTimestamp($this->fs->lastModified($path));
        return Response::make($this->fs->get($path), 200, ['Content-Type' => 'image/svg+xml'])
            ->setMaxAge(15552000)
            ->setLastModified($modified)
            ->setExpires($expires);
    }

    /**
     * @param TdfmFile $tdfmFile
     * @throws FileNotFoundException
     */
    private function generateSizes(TdfmFile $tdfmFile)
    {
        $name = preg_replace('/(\w+)(.)(\w+)/', '${1}', $tdfmFile->name);
        $ext = preg_replace('/(\w+)(.)(\w+)/', '${3}', $tdfmFile->name);
        foreach (Config::get('tdfm.sizes') as $key => $size) {
            $image = FmImage::fromString(Storage::disk('tdfm_original')->get($tdfmFile->name));

            $width = $image->getWidth() < $size ? $image->getWidth() : $size;
            if ($key !== 'xs') {
                $image->resize($width);
            } else {
                $image->thumbnail($width, $width);
            }
            $image->autoOrient();
            Storage::disk('tdfm_generated')->put(
                sprintf('%s/%s/%s.%s', $tdfmFile->id, $key, $name, $ext),
                $image->toString($tdfmFile->file_mime, Config::get('tdfm.quality'))
            );

            if (Config::get('tdfm.generate_webp')) {
                Storage::disk('tdfm_generated')->put(
                    sprintf('%s/%s/%s.%s', $tdfmFile->id, $key, $name, 'webp'),
                    $image->toString('image/webp', Config::get('tdfm.quality'))
                );
            }
        }
    }
}
